import 'dart:io';
import 'package:scalator/src/setting/collector_app_bar.dart';
import 'package:scalator/src/setting/collector_drawer.dart';
import 'package:scalator/src/setting/collector_footer.dart';
import 'package:dart_bones/dart_bones.dart';
import 'package:flutter/material.dart';

import '../page/page_controller_collector.dart';

enum RedrawReason {
  custom,
  callback,
  fetchList,
  fetchRecord,
  redraw,
  setError,
}

abstract class ErrorHandlerCollector {
  /// Handles a critical error: "translate" a technical message into a user
  /// readable message...
  /// Returns false (for chaining)
  bool criticalError(String errorMessage,
      {String caller, String customParameter});
}

abstract class FooterInterface {
  Widget widget(PageControllerCollector controller);
}

typedef AppBarBuilder = Function(String);
typedef DrawerBuilder = Function(dynamic);
typedef FooterBuilder = FooterInterface Function();
enum Layout { Portrait, Landscape, Square }

class DirInfo {
  final String label;
  String path;
  DirInfo(this.label, this.path);
}

class ImageDimensions {
  static final regExpr = RegExp(r'(\d+)\s*x\s*(\d+)');
  final int maxWidth;
  final int maxHeight;
  ImageDimensions(this.maxWidth, this.maxHeight);
  ImageDimensions.fromString(String input)
      : this(int.parse(regExpr.firstMatch(input)?.group(1)),
            int.parse(regExpr.firstMatch(input).group(2)));
}

/// Speicherung globaler Resourcen. Ist ein Singleton.
class ApplicationData implements ErrorHandlerCollector {
  static ApplicationData _instance;
  final BaseConfiguration configuration;
  final BaseLogger logger;
  final version = '0.1.0';

  /// Signature: AppBar func(String title)
  final AppBarBuilder appBarBuilder;
  final DrawerBuilder drawerBuilder;
  final FooterBuilder footerBuilder;
  DirInfo sourceDirectory = DirInfo('Quellverzeichnis', Directory.current.path);
  DirInfo targetDirectory = DirInfo('Zielverzeichnis', Directory.systemTemp.path);
  String extensions;
  final messageController = TextEditingController();
  ImageDimensions portrait;
  ImageDimensions landscape;
  ImageDimensions square;
  int quality = 75;
  PageControllerCollector currentPageController;
  final fileList = <String>[];
  int convertedBytes = 0;
  int processedBytes = 0;
  int processedFiles = 0;
  int copiedFiles = 0;

  /// <page_full_name>: <last_error_message>
  final _lastErrorMessageMap = <String, String>{};
  static ApplicationData create(
      BaseConfiguration configuration, BaseLogger logger) {
    _instance = ApplicationData.internal(configuration, CollectorAppBar.builder,
        CollectorDrawer.builder, CollectorFooter.builder, logger);
    return _instance;
  }

  factory ApplicationData() => _instance;

  /// [configuration]: allgemeine Konfigurationen
  /// [appBarBuilder] eine "Factory" zur Erzeugung des "Hamburger-Menüs"
  /// [footerBuilder] eine "Factory" zur Erzeugung des Footer-Bereichs
  ApplicationData.internal(this.configuration, this.appBarBuilder,
      this.drawerBuilder, this.footerBuilder, this.logger);

  /// Speichert den letzten Fehler der [page]. [message] kann null sein (kein Fehler).
  /// Wenn [page] null: Die aktuelle Seite wird genommen.
  void setLastErrorMessage(String page, String message) {
    if (page == null && currentPageController != null) {
      page = currentPageController.pageName;
    }
    if (page != null) {
      _lastErrorMessageMap[page] = message;
    }
  }

  /// Liefert den letzten schweren Fehler von Seite [page] oder null.
  String lastErrorMessage(String page) {
    final rc = _lastErrorMessageMap.containsKey(page)
        ? _lastErrorMessageMap[page]
        : null;
    return rc;
  }

  @override
  bool criticalError(String errorMessage,
      {String caller, String customParameter}) {
    if (caller == 'RestPersistance.runRequest') {
      if (errorMessage.contains('SocketException: OS Error')) {
        setLastErrorMessage(null, 'Keine Verbindung zum Server');
      }
    }
    return false;
  }
}
